/* getrecords.p 
   Return a batch of records for any table 
   Author: Peter van Dam, Future Proof Software */

ROUTINE-LEVEL ON ERROR UNDO, THROW.

DEFINE INPUT PARAMETER tableName AS CHAR NO-UNDO.
DEFINE INPUT PARAMETER wherePhrase AS CHAR NO-UNDO.
DEFINE INPUT PARAMETER fieldList AS CHAR NO-UNDO.
DEFINE INPUT PARAMETER batchSize AS INT NO-UNDO.
DEFINE INPUT PARAMETER batchNumber AS INT NO-UNDO.
DEFINE OUTPUT PARAMETER lMore AS LOGICAL NO-UNDO.
DEFINE OUTPUT PARAMETER TABLE-HANDLE hTable.


DEF VAR currentBatch AS INT INITIAL 1 NO-UNDO.
DEF VAR currentRow AS INT NO-UNDO.
DEF VAR iField AS INTEGER NO-UNDO. 
DEF VAR hField AS HANDLE NO-UNDO.
DEF VAR hBuffer AS HANDLE NO-UNDO.
DEF VAR hQuery AS HANDLE NO-UNDO.
DEF VAR hDefault AS HANDLE NO-UNDO.

/* create a dynamic temp-table */
CREATE BUFFER hBuffer FOR TABLE tableName.
CREATE TEMP-TABLE hTable.

IF fieldList = "" OR fieldList = "*" THEN
  hTable:CREATE-LIKE(hBuffer).
ELSE DO iField = 1 TO NUM-ENTRIES(fieldList):
  ASSIGN hField = hBuffer:BUFFER-FIELD(ENTRY(iField,fieldList)) NO-ERROR.
  IF VALID-HANDLE(hField) THEN
    hTable:ADD-LIKE-FIELD(hField:NAME,hField). 
END.


hTable:TEMP-TABLE-PREPARE("myTable").
hDefault = hTable:DEFAULT-BUFFER-HANDLE.
hTable:PRIVATE-DATA = PROGRAM-NAME(1).

CREATE QUERY hQuery.
hQuery:SET-BUFFERS(hBuffer).
hQuery:QUERY-PREPARE(SUBSTITUTE("FOR EACH &1 NO-LOCK WHERE &2", 
                                hBuffer:NAME,
                                wherePhrase)).
hQuery:QUERY-OPEN().

/* return the requested batch of records */
DO WHILE hQuery:GET-NEXT():

  ASSIGN currentRow = currentRow + 1.

  IF currentRow > batchSize THEN DO:
    ASSIGN currentBatch = currentBatch + 1
           currentRow = 1.
    IF currentBatch > batchNumber THEN DO:
      ASSIGN lMore = TRUE. /* More records available */
      LEAVE.
    END. 
  END.

  IF currentBatch = batchNumber THEN DO:
    hDefault:BUFFER-CREATE().
    hDefault:BUFFER-COPY(hBuffer).
  END.

END.


FINALLY:
  DELETE OBJECT hBuffer NO-ERROR.
  DELETE OBJECT hQuery NO-ERROR.
  DELETE OBJECT hTable NO-ERROR.
END FINALLY. 

