/* getxmlcustomers.p 
   Return a batch of records for any table 
   Author: Peter van Dam, Future Proof Software */

ROUTINE-LEVEL ON ERROR UNDO, THROW.

{src/web2/wrap-cgi.i}

RUN outputContentType IN web-utilities-hdl ("text/xml":U).

DEF VAR batchSize AS INT INITIAL 20 NO-UNDO.
DEF VAR batchNumber AS INT INITIAL 1 NO-UNDO.
DEF VAR fieldList AS CHAR INITIAL "custNum,Name,City,Country" NO-UNDO.
DEF VAR wherePhrase AS CHAR NO-UNDO.
DEF VAR hTable AS HANDLE NO-UNDO.
DEF VAR lMore AS LOGICAL NO-UNDO.

DEF VAR hDoc AS HANDLE NO-UNDO.
DEF VAR hRoot AS HANDLE NO-UNDO.
DEF VAR iChild AS INT NO-UNDO.
DEF VAR hElement AS HANDLE NO-UNDO.
DEF VAR hText AS HANDLE NO-UNDO.

CREATE X-DOCUMENT hdoc.
CREATE X-NODEREF hRoot.
CREATE X-NODEREF hElement.
CREATE X-NODEREF hText.
ASSIGN hDoc = WEB-CONTEXT:X-DOCUMENT.
hDoc:GET-DOCUMENT-ELEMENT(hRoot).

DO iChild = 1 TO hRoot:NUM-CHILDREN:
  hRoot:GET-CHILD(hElement,iChild).
  hElement:GET-CHILD(hText,1).
  CASE hElement:NAME:
    WHEN "batchSize" THEN
      ASSIGN batchSize = INTEGER(hText:NODE-VALUE) NO-ERROR.
    WHEN "batchNumber" THEN
      ASSIGN batchNumber = INTEGER(hText:NODE-VALUE) NO-ERROR.
    WHEN "fieldList" THEN
      ASSIGN fieldList = hText:NODE-VALUE NO-ERROR.
    WHEN "wherePhrase" THEN
      ASSIGN wherePhrase = url-decode(hText:NODE-VALUE) NO-ERROR.
  END CASE.
END. /* for each tag */

DELETE OBJECT hText.
DELETE OBJECT hElement.
DELETE OBJECT hRoot.
DELETE OBJECT hdoc.

RUN getrecords.p (INPUT "customer",
                  INPUT wherePhrase,
                  INPUT fieldList,
                  INPUT batchSize,
                  INPUT batchNumber,
                  OUTPUT lMore,
                  OUTPUT TABLE-HANDLE hTable).
                  
PAUSE RANDOM(2,5).

hTable:WRITE-XML("STREAM", "WEBSTREAM").
hTable:WRITE-XML("FILE", "mytable.xml").

DELETE OBJECT hTable. 

CATCH errorObject AS Progress.Lang.SysError:

  DEFINE VARIABLE iMessage# AS INT NO-UNDO.

  DEFINE VARIABLE hSAXWriter AS HANDLE.
  CREATE SAX-WRITER hSAXWriter.
  hSAXWriter:FORMATTED = TRUE.
  hSAXWriter:SET-OUTPUT-DESTINATION("STREAM", "WEBSTREAM").
  hSAXWriter:START-DOCUMENT().
  hSAXWriter:START-ELEMENT("error").

  DO iMessage# = 1 TO errorObject:numMessages:
    hSAXWriter:WRITE-DATA-ELEMENT("errorMessage", errorObject:getMessage(iMessage#)).
  END.
  
  hSAXWriter:END-ELEMENT("error").
  hSAXWriter:END-DOCUMENT().

  DELETE OBJECT hSAXWriter.

  DELETE OBJECT hTable NO-ERROR. 
  
  DELETE OBJECT errorObject. 

END CATCH.
